@@ -12,7 +12,7 @@ except ImportError:  # pragma: no cover
     # ignore mypy error about incompatible imports
     from importlib_metadata import version, PackageNotFoundError  # type: ignore
 
-from os import environ, path
+from os import path
 from typing import Any, Dict
 
 from sphinx.application import Sphinx
@@ -25,26 +25,17 @@ except PackageNotFoundError:  # pragma: no cover
 
 
 def setup(app: "Sphinx") -> Dict[str, Any]:
-    """Register the theme and its extensions wih Sphinx.
-
-    The setup function of this theme accomplishes the following:
-
-    - add the HTML theme
-    - activate the ``sphinxawesome.sampdirective`` extension
-    - activate the ``sphinxawesome_theme.highlighting`` extension
-    - activate the ``sphinxawesome_theme.html_translator`` extension
-    - add the ``AdmonitionID`` as post-transform
-    - execute the ``post_process_html`` code when the build has finished
-    """
+    """Register the theme and its extensions wih Sphinx."""
     app.add_html_theme("sphinxawesome_theme", path.abspath(path.dirname(__file__)))
-    app.setup_extension("sphinxawesome.sampdirective")
+    app.add_config_value("html_awesome_postprocessing", True, "html")
+    app.add_config_value("html_awesome_html_translator", True, "html")
     app.setup_extension("sphinxawesome_theme.highlighting")
-    app.setup_extension("sphinxawesome_theme.html_translator")
-    app.setup_extension("sphinxawesome_theme.admonition_ids")
-    app.setup_extension("sphinxawesome_theme.jinja_filters")
+    app.setup_extension("sphinxawesome_theme.jinja_functions")
 
-    # if this environment variable is defined, skip the postprocessing
-    if "SPHINX_AWESOME_THEME_NO_POSTPROCESSING" not in environ:
+    if app.config.html_awesome_html_translator:
+        app.setup_extension("sphinxawesome_theme.html_translator")
+
+    if app.config.html_awesome_postprocessing:
         app.setup_extension("sphinxawesome_theme.postprocess")
 
     return {
