@@ -1,26 +1,27 @@
 """Post-process the HTML produced by Sphinx.
 
-Modifications that can be done on the finished HTML
-are better done using BeautifulSoup. Here, a simple
-pipeline is defined for reading all HTML files,
-parsing them with BeautifulSoup and perform a chain
-of actions on the tree in place.
+Some modifications can be done more easily on the finished HTML.
+
+This module defines a simple pipeline:
+
+1. Read all HTML files
+2. Parse them with `BeautifulSoup`
+3. Perform a chain of actions on the tree in place
+
 See the `_modify_html()` function for the list of
 transformations.
 
-Note: If you add elements to the HTML, add traditional
-classes instead of using Tailwind's utility classes.
-This is because this Python file is not processed by
-PurgeCSS.
+Note: This file is not processed by Webpack; don't use Tailwind utility classes.
+They might not show up in the final CSS.
 
-:copyright: Copyright 2020, Kai Welke.
+:copyright: Copyright Kai Welke.
 :license: MIT, see LICENSE.
 """
 
 import os
 from typing import Any, Dict, List, Optional
 
-from bs4 import BeautifulSoup
+from bs4 import BeautifulSoup, Comment
 from sphinx.application import Sphinx
 from sphinx.util import logging
 
@@ -33,7 +34,7 @@ logger = logging.getLogger(__name__)
 def _get_html_files(outdir: str) -> List[str]:
     """Get a list of HTML files."""
     html_list = []
-    for root, _dirs, files in os.walk(outdir):
+    for root, _, files in os.walk(outdir):
         html_list.extend(
             [os.path.join(root, file) for file in files if file.endswith(".html")]
         )
@@ -52,6 +53,7 @@ def _collapsible_nav(tree: BeautifulSoup) -> None:
     captured separately between the icon and the link.
     """
     for link in tree.select(".nav-toc a"):
+        link["data-action"] = "click->sidebar#close"
         # Don't add the nav-link class twice (#166)
         if "nav-link" not in link.parent.get("class", []):
             # First, all links should be wrapped in a div.nav-link
@@ -60,7 +62,12 @@ def _collapsible_nav(tree: BeautifulSoup) -> None:
             # has any sibling elements (a ``ul`` in the navigation menu)
             if link.parent.next_sibling:
                 # create the icon
-                svg = BeautifulSoup(ICONS["chevron_right"], "html.parser")
+                svg = BeautifulSoup(ICONS["chevron_right"], "html.parser").svg
+                svg["tabindex"] = "0"
+                svg["class"] = ["expand"]
+                svg[
+                    "data-action"
+                ] = "click->sidebar#expand keydown->sidebar#expandKeyPressed"
                 link.insert_before(svg)
 
 
@@ -81,14 +88,48 @@ def _remove_span_pre(tree: BeautifulSoup) -> None:
         span.unwrap()
 
 
-def _modify_html(html_filename: str) -> None:
+def _remove_empty_toctree(tree: BeautifulSoup) -> None:
+    """Remove empty toctree divs.
+
+    If you include a `toctree` with the `hidden` option,
+    an empty `div` is inserted. Remove them.
+    The empty `div` contains a single `end-of-line` character.
+    """
+    for div in tree("div", class_="toctree-wrapper"):
+        children = list(div.children)
+        if len(children) == 1 and not children[0].strip():
+            div.extract()
+
+
+def _headerlinks(tree: BeautifulSoup) -> None:
+    """Enhance the headerlink experience."""
+    for link in tree("a", class_="headerlink"):
+        link["data-controller"] = "clipboard"
+        link["data-action"] = "click->clipboard#copyHeaderLink"
+        link["aria-label"] = "Click to copy this link"
+        del link["title"]
+        link["class"].extend(["tooltipped", "tooltipped-ne"])
+
+
+def _external_links(tree: BeautifulSoup) -> None:
+    """Add `rel="nofollow noopener"` to external links."""
+    for link in tree("a", class_="reference external"):
+        link["rel"] = "nofollow noopener"
+
+
+def _strip_comments(tree: BeautifulSoup) -> None:
+    """Remove HTML comments from documents."""
+    comments = tree.find_all(string=lambda text: isinstance(text, Comment))
+    for c in comments:
+        c.extract()
+
+
+def _modify_html(html_filename: str, app: Sphinx) -> None:
     """Modify a single HTML document.
 
-    The HTML document is parsed into a BeautifulSoup tree.
-
-    The modifications are performed in order and in place.
-
-    After these modifications, the HTML is written into a file,
+    1. The HTML document is parsed into a BeautifulSoup tree.
+    2. The modifications are performed in order and in place.
+    3. After these modifications, the HTML is written into a file,
     overwriting the original file.
     """
     with open(html_filename, encoding="utf-8") as html:
@@ -97,6 +138,11 @@ def _modify_html(html_filename: str) -> None:
     _expand_current(tree)
     _collapsible_nav(tree)
     _remove_span_pre(tree)
+    _remove_empty_toctree(tree)
+    _external_links(tree)
+    if app.config.html_awesome_headerlinks:
+        _headerlinks(tree)
+    _strip_comments(tree)
 
     with open(html_filename, "w") as out_file:
         out_file.write(str(tree))
@@ -116,12 +162,13 @@ def post_process_html(app: Sphinx, exc: Optional[Exception]) -> None:
         html_files = _get_html_files(app.outdir)
 
         for doc in html_files:
-            _modify_html(doc)
+            _modify_html(doc, app)
 
 
 def setup(app: "Sphinx") -> Dict[str, Any]:
     """Set this up as internal extension."""
     app.connect("build-finished", post_process_html)
+    app.add_config_value("html_awesome_headerlinks", True, "html")
 
     return {
         "version": __version__,
